##########################################
## Cosmetics Europe Skin Sensitization
## Defined Approach Evaluation
## Nicole Kleinstreuer
## nicole.kleinstreuer@nih.gov
##
## Kao Defined Approaches 
## STS: Tiered system for predicting sensitizing potential and potency of a substance
## ITS: Score-based battery system for predicting sensitizing potential and potency of a substance
##
## Nukada et al. 2013 (101 chems), Takenouchi et al. 2015 (139 chems)
##
##Replace ".MNC" with ".GM" or appropriate metric to change LLNA EC3 summary comparison
##########################################

rm(list=ls())

##Change directory string 
input.dir <- "/Users/nkleinstreuer/Documents/Cosmetics Europe/"
date.string<-Sys.Date()
setwd(input.dir)

library(gdata)
library(caret)
library(e1071)

##########################################
#Read Cosmetic Europe in vitro data and DEREK predictions
CE<-read.table("CosEU_invitro_insilico_128chems_Jan2017.txt",header=TRUE,sep="\t")
CE$CASRN<-trim(CE$CASRN)
CE<-CE[order(CE$Num),]

##########################################
#Read in vivo LLNA and human data
IV<-read.table("CosEU_invivo_128chems_March2017.txt",header=TRUE,sep="\t")
IV$CASRN<-trim(IV$CASRN)
IV<-IV[order(IV$Num),]

##########################################
##Read data from Takenouchi et al. 2015
#KaoDB<-read.csv("TakenouchiDB_computable.csv",header=TRUE)
##Look for overlap
#KaoDB$CASRN<-trim(KaoDB$CASRN)
#length(intersect(CE$CASRN,KaoDB$CASRN))
### 74 chemicals overlap

##########################################
#Apply Kao STS (Nukada et al. 2013, Takenouchi et a. 2015) based only on hCLAT and DPRA
CE$Kao.STS.Call<-0
CE$Kao.STS.Call[CE$hCLAT.MIT<5000]<-1
CE$Kao.STS.Call[CE$DPRA.Call==1]<-1
#Remove chemicals with NA for hCLAT or DPRA
CE$Kao.STS.Call[is.na(CE$hCLAT.MIT) | is.na(CE$DPRA.Call)]<-NA
#Potency scoring
CE$Kao.STS.Pot<-"Non"
CE$Kao.STS.Pot[CE$hCLAT.MIT<10]<-"Strong"
CE$Kao.STS.Pot[CE$hCLAT.MIT>10 & CE$hCLAT.MIT<5000]<-"Weak"
CE$Kao.STS.Pot[CE$hCLAT.MIT>5000 & CE$DPRA.Call==1]<-"Weak"
CE$Kao.STS.Pot[is.na(CE$hCLAT.MIT) | is.na(CE$DPRA.Call)]<-NA

##########################################
#Results for all chemicals (n=127)
#Create confusion matrix to compare in vitro call and LLNA result for STS 
confusionMatrix(data=CE$Kao.STS.Call, reference=IV$LLNA.Call.MNC, positive="1")
#Create confusion matrix to compare in vitro potency class and LLNA potency result for STS 
IV$LLNA.potency.3class.MNC<-factor(IV$LLNA.potency.3class.MNC, levels=c("Non","Weak","Strong"))
CE$Kao.STS.Pot<-factor(CE$Kao.STS.Pot, levels=c("Non","Weak","Strong"))
confusionMatrix(data=CE$Kao.STS.Pot, reference=IV$LLNA.potency.3class.MNC)

#Create confusion matrix to compare in vitro call and human result for STS 
confusionMatrix(data=CE$Kao.STS.Call, reference=IV$human.Call, positive="1")
#Create confusion matrix to compare in vitro potency class and human potency result for STS 
IV$human.potency.3class<-factor(IV$human.potency.3class, levels=c("Non","Weak","Strong"))
confusionMatrix(data=CE$Kao.STS.Pot, reference=IV$human.potency.3class)

##########################################
#Results for "test set" chemicals (n=53)
CEt<-CE[CE$CASRN %in% setdiff(CE$CASRN,KaoDB$CASRN),]
IVt<-IV[IV$CASRN %in% setdiff(CE$CASRN,KaoDB$CASRN),]
#Create confusion matrix to compare in vitro call and LLNA result for STS 
confusionMatrix(data=CEt$Kao.STS.Call, reference=IVt$LLNA.Call.MNC, positive="1")
#Create confusion matrix to compare in vitro potency class and LLNA potency result for STS 
confusionMatrix(data=CEt$Kao.STS.Pot, reference=IVt$LLNA.potency.3class.MNC)
#Create confusion matrix to compare in vitro call and human result for STS 
confusionMatrix(data=CEt$Kao.STS.Call, reference=IVt$human.Call, positive="1")
#Create confusion matrix to compare in vitro potency class and human potency result for STS 
confusionMatrix(data=CEt$Kao.STS.Pot, reference=IVt$human.potency.3class)

##########################################
#Mark overlap set chemicals
CE$Kao.STS.ITS_Overlap<-0
CE$Kao.STS.ITS_Overlap[CE$CASRN %in% intersect(CE$CASRN,KaoDB$CASRN)]<-1

##########################################
##########################################
#Apply Kao ITS (Takenouchi et al. 2015)
#Check for overlap with DEREK training set
#summary(CE$DEREK.Training.set) shows 17 chemicals in DEREK training set, all were in Takenouchi set
CE$Kao.ITS2015.Score<-0
#hCLAT in vitro data converted to scores
CE$Kao.ITS2015.Score.hC<-0
CE$Kao.ITS2015.Score.hC[CE$hCLAT.MIT>0 & CE$hCLAT.MIT<10]<-3
CE$Kao.ITS2015.Score.hC[CE$hCLAT.MIT>10 & CE$hCLAT.MIT<150]<-2
CE$Kao.ITS2015.Score.hC[CE$hCLAT.MIT>150 & CE$hCLAT.MIT<5000]<-1
#DPRA in vitro data converted to scores
CE$Kao.ITS2015.Score.DPRA<-0
CE$Kao.ITS2015.Score.DPRA[CE$DPRA.mean>42.47]<-3
CE$Kao.ITS2015.Score.DPRA[CE$DPRA.mean>22.62 & CE$DPRA.mean<42.47]<-2
CE$Kao.ITS2015.Score.DPRA[CE$DPRA.mean>6.376 & CE$DPRA.mean<22.62]<-1
##LYSINE CO-ELUTION CONDITIONALITY
for (i in 1:dim(CE)[1]){
  if (CE[i,"DPRA.percLysdep"]<0 & !is.na(CE[i,"DPRA.percLysdep"])){
    cat(i)
    cat("Lysine co-elution, using cysteine depletion only")
    CE[i,"Kao.ITS2015.Score.DPRA"]<-0
    if (CE[i,"DPRA.percCysdep"]>98.23) {CE[i,"Kao.ITS2015.Score.DPRA"]<-3}
    if (CE[i,"DPRA.percCysdep"]<98.24 & CE[i,"DPRA.percCysdep"]>23.08) {CE[i,"Kao.ITS2015.Score.DPRA"]<-2}
    if (CE[i,"DPRA.percCysdep"]<23.09 & CE[i,"DPRA.percCysdep"]>13.88) {CE[i,"Kao.ITS2015.Score.DPRA"]<-1}
  }
}
#DEREK Nexus predictions converted to scores
CE$Kao.ITS2015.Score.Der<-0
CE$Kao.ITS2015.Score.Der[CE$DEREK.Skin.Sensitisation=="Positive"]<-1
#Combine scores 
CE$Kao.ITS2015.Score<-rowSums(CE[,c("Kao.ITS2015.Score.hC","Kao.ITS2015.Score.DPRA","Kao.ITS2015.Score.Der")])
#Remove chemicals with NA for Derek or hCLAT or DPRA
CE$Kao.ITS2015.Score[is.na(CE$hCLAT.MIT) | is.na(CE$DPRA.Call)]<-NA
CE$Kao.ITS2015.Score[CE$DEREK.Skin.Sensitisation=="na"]<-NA

#Assign potency class base on score
CE$Kao.ITS2015.Pot<-"Non"
CE$Kao.ITS2015.Pot[CE$Kao.ITS2015.Score==7]<-"Strong"
CE$Kao.ITS2015.Pot[CE$Kao.ITS2015.Score<7 & CE$Kao.ITS2015.Score>1]<-"Weak"
#Remove chemicals with NA for Derek or hCLAT or DPRA
CE$Kao.ITS2015.Pot[is.na(CE$hCLAT.MIT) | is.na(CE$DPRA.Call)]<-NA
CE$Kao.ITS2015.Pot[CE$DEREK.Skin.Sensitisation=="na"]<-NA

#Positive (Strong + Weak) or Negative (Non) based on score
CE$Kao.ITS2015.Call<-0
CE$Kao.ITS2015.Call[CE$Kao.ITS2015.Pot=="Strong" | CE$Kao.ITS2015.Pot=="Weak"]<-1
#Remove chemicals with NA for Derek or hCLAT or DPRA
CE$Kao.ITS2015.Call[is.na(CE$hCLAT.MIT) | is.na(CE$DPRA.Call)]<-NA
CE$Kao.ITS2015.Call[CE$DEREK.Skin.Sensitisation=="na"]<-NA

#Create confusion matrix to compare in vitro call and LLNA result for ITS 
confusionMatrix(data=CE$Kao.ITS2015.Call, reference=IV$LLNA.Call.MNC, positive="1")
#Create confusion matrix to compare in vitro potency class and LLNA potency result for ITS 
CE$Kao.ITS2015.Pot<-factor(CE$Kao.ITS2015.Pot, levels=c("Non","Weak","Strong"))
confusionMatrix(data=CE$Kao.ITS2015.Pot, reference=IV$LLNA.potency.3class.MNC)

#Create confusion matrix to compare in vitro call and human result for ITS 
confusionMatrix(data=CE$Kao.ITS2015.Call, reference=IV$human.Call, positive="1")
#Create confusion matrix to compare in vitro potency class and LLNA potency result for ITS 
confusionMatrix(data=CE$Kao.ITS2015.Pot, reference=IV$human.potency.3class)

##########################################
#Results for "test set" chemicals (n=53)
CEt<-CE[CE$CASRN %in% setdiff(CE$CASRN,KaoDB$CASRN),]
IVt<-IV[IV$CASRN %in% setdiff(CE$CASRN,KaoDB$CASRN),]
#Create confusion matrix to compare in vitro call and LLNA result for ITS2015 
confusionMatrix(data=CEt$Kao.ITS2015.Call, reference=IVt$LLNA.Call.MNC, positive="1")
#Create confusion matrix to compare in vitro potency class and LLNA potency result for ITS2015 
confusionMatrix(data=CEt$Kao.ITS2015.Pot, reference=IVt$LLNA.potency.3class.MNC)
#Create confusion matrix to compare in vitro call and human result for ITS2015 
confusionMatrix(data=CEt$Kao.ITS2015.Call, reference=IVt$human.Call, positive="1")
#Create confusion matrix to compare in vitro potency class and human potency result for ITS2015 
confusionMatrix(data=CEt$Kao.ITS2015.Pot, reference=IVt$human.potency.3class)

##########################################
##########################################
#Additional ITS(v1): Nukada et al. 2013
##########################################
#Apply Kao ITS2013 (Nukada et al. 2013)
CE$Kao.ITS2013.Score<-0
#hCLAT in vitro data converted to scores
CE$Kao.ITS2013.Score.hC<-0
CE$Kao.ITS2013.Score.hC[CE$hCLAT.MIT>0 & CE$hCLAT.MIT<10]<-2
CE$Kao.ITS2013.Score.hC[CE$hCLAT.MIT>10 & CE$hCLAT.MIT<5000]<-1
#DPRA in vitro data converted to scores
CE$Kao.ITS2013.Score.DPRA<-0
CE$Kao.ITS2013.Score.DPRA[CE$DPRA.mean>22.62]<-2
CE$Kao.ITS2013.Score.DPRA[CE$DPRA.mean>6.376 & CE$DPRA.mean<22.62]<-1
#DEREK Nexus predictions converted to scores
CE$Kao.ITS2013.Score.Der<-0
CE$Kao.ITS2013.Score.Der[CE$DEREK.Skin.Sensitisation=="Positive"]<-1
CE$Kao.ITS2013.Score.Der[CE$DEREK.Skin.Sensitisation=="na"]<-NA
#Combine scores 
CE$Kao.ITS2013.Score<-rowSums(CE[,c("Kao.ITS2013.Score.hC","Kao.ITS2013.Score.DPRA","Kao.ITS2013.Score.Der")])
#Remove chemicals with NA for Derek or hCLAT or DPRA
CE$Kao.ITS2013.Score[is.na(CE$hCLAT.MIT) | is.na(CE$DPRA.Call)]<-NA
CE$Kao.ITS2013.Score[CE$DEREK.Skin.Sensitisation=="na"]<-NA
#Assign potency class base on score
CE$Kao.ITS2013.Pot<-"Non"
CE$Kao.ITS2013.Pot[CE$Kao.ITS2013.Score==5]<-"Strong"
CE$Kao.ITS2013.Pot[CE$Kao.ITS2013.Score<5 & CE$Kao.ITS2013.Score>1]<-"Weak"
#Remove chemicals with NA for Derek or hCLAT or DPRA
CE$Kao.ITS2013.Pot[is.na(CE$hCLAT.MIT) | is.na(CE$DPRA.Call)]<-NA
CE$Kao.ITS2013.Pot[CE$DEREK.Skin.Sensitisation=="na"]<-NA
#Positive (Strong + Weak) or Negative (Non) based on score
CE$Kao.ITS2013.Call<-0
CE$Kao.ITS2013.Call[CE$Kao.ITS2013.Pot=="Strong" | CE$Kao.ITS2013.Pot=="Weak"]<-1
#Remove chemicals with NA for Derek or hCLAT or DPRA
CE$Kao.ITS2013.Call[is.na(CE$hCLAT.MIT) | is.na(CE$DPRA.Call)]<-NA
CE$Kao.ITS2013.Call[CE$DEREK.Skin.Sensitisation=="na"]<-NA

##########################################
#Create confusion matrix to compare in vitro call and LLNA result for ITS 
confusionMatrix(data=CE$Kao.ITS2013.Call, reference=IV$LLNA.Call.MNC, positive="1")
#Create confusion matrix to compare in vitro potency class and LLNA potency result for ITS 
confusionMatrix(data=CE$Kao.ITS2013.Pot, reference=IV$LLNA.potency.3class.MNC)
#Create confusion matrix to compare in vitro call and human result for ITS 
confusionMatrix(data=CE$Kao.ITS2013.Call, reference=IV$human.Call, positive="1")
#Create confusion matrix to compare in vitro potency class and human potency result for ITS 
confusionMatrix(data=CE$Kao.ITS2013.Pot, reference=IV$human.potency.3class)