##########################################
## Cosmetics Europe Skin Sensitization
## Defined Approach Evaluation
## Nicole Kleinstreuer
## nicole.kleinstreuer@nih.gov
##
## ICCVAM SSWG SVM Model
##
## LLNA: Strickland et al. 2015
## (94 chemical training set, 26 chemical test set)
##
## Human: Strickland et al. 2016
## (72 chemical training set, 24 chemical test set)
##
##Replace ".MNC" with ".GM" or appropriate metric to change LLNA EC3 summary comparison
##########################################

rm(list=ls())
##Change directory string 
input.dir <- "C:/Users/Nicole/Documents/Cosmetics Europe/Kleinstreuer_etal_CE_DA_code_July2017/"
date.string<-Sys.Date()
setwd(input.dir)

library(gdata)
library(caret)
library(e1071)


##########################################
#Read Cosmetic Europe in vitro data, physchem properties, OECD Toolbox predictions
CE<-read.table("CosEU_invitro_insilico_128chems_Jan2017.txt",header=TRUE,sep="\t")
CE$CASRN<-trim(CE$CASRN)
CE<-CE[order(CE$Num),]
#Format data (match feature names from ICCVAM model)
CE$hCLAT<-CE$hCLAT.Call

##########################################
#Read in vivo LLNA and human data
IV<-read.table("CosEU_invivo_128chems_March2017.txt",header=TRUE,sep="\t")
IV$CASRN<-trim(IV$CASRN)
IV<-IV[order(IV$Num),]
##Format data (LLNA call based on which EC3 metric)
IV$LLNA.POS.NEG<-"POS"
IV$LLNA.POS.NEG[IV$LLNA.Call.MNC==0]<-"NEG"

##########################################
#Build ICCVAM SVM LLNA model
# Read in the training data with 94 chemicals and 17 columns
LLNATraining <- read.table("ICCVAM-LLNA-Sensitizers-NonSensitizers-Training.txt", header=T, sep="\t", as.is=T )
# Use hCLAT, OECD and six properties to build models.
trainingdata<-LLNATraining[, c("MW", "LogP", "LogS", "LogVP", "MP", "BP", "hCLAT", "OECD")]
# There are 26 negatives and 68 positives for training set
TrainingClass <- factor(c( rep("NEG", 26), rep("POS", 68)))
# Set a weight for balancing the negative and positive classes, where "7" is an optimal value #from a series of trials from 1 to 10.
wts <- 7/ table(TrainingClass)
LLNABinSVM <- svm(trainingdata, TrainingClass, cost = 400, gamma = 0.081, class.weights = wts)

# Read in the test data with 26 chemicals and 17 columns
LLNATest <- read.table("ICCVAM-LLNA-Sensitizers-NonSensitizers-Test.txt", header=T, sep="\t", as.is=T )
# Use hCLAT, OECD and six properties to build models.
testdata<-LLNATest[, c("MW", "LogP", "LogS", "LogVP", "MP", "BP", "hCLAT", "OECD")]
# There are 7 negatives and 19 positives for test set
TestClass <- factor(c( rep("NEG", 7), rep("POS", 19)))


##########################################
#Reproduce results from Strickland et al. 2015
#Predict the training set
PredTraining<-predict(LLNABinSVM, trainingdata)
table(PredTraining, TrainingClass)
#Predict the test set
PredTest<-predict(LLNABinSVM, testdata)
table(PredTest, TestClass)

##########################################
#Look for overlap with model training set
length(intersect(CE$CASRN,LLNATraining$CASRN))
# 57 chemicals overlap with training set
length(intersect(CE$CASRN,LLNATest$CASRN))
# 17 chemicals overlap with test set

##########################################
#Results for all chemicals (n=120)
#Create confusion matrix to compare in vitro call and LLNA result
Newdata1<-CE[, c("Num","CASRN","Chemical","hCLAT", "OECD", "MW", "LogP", "LogS", "LogVP", "MP", "BP")]
#Check for NA in input variables
Newdata1<-na.omit(Newdata1)
Newdata<-Newdata1[, c("MW", "LogP", "LogS", "LogVP", "MP", "BP", "hCLAT", "OECD")]
IVdata<-IV[which(IV$Num %in% Newdata1$Num),]
#Define endpoint (LLLNA or human)
NewClass<-factor(IVdata[,"LLNA.POS.NEG"])
#Run SVM model and evaluate prediction
PredNew<-predict(LLNABinSVM, Newdata)
confusionMatrix(data=PredNew, reference=NewClass, positive="POS")

LLNASVM<-cbind(Newdata1,PredNew,NewClass)
names(LLNASVM)[12:13]<-c("ICCVAM_SVM_LLNA_Prediction","LLNA_Class")
CE<-merge(CE,LLNASVM,all.x=T)

##########################################
#Results for CE "test set" chemicals (n=63)
CEt<-CE[CE$CASRN %in% setdiff(CE$CASRN,LLNATraining$CASRN),]
IVt<-IV[IV$CASRN %in% setdiff(CE$CASRN,LLNATraining$CASRN),]
Newdata1<-CEt[, c("Num","CASRN","Chemical","hCLAT", "OECD", "MW", "LogP", "LogS", "LogVP", "MP", "BP")]
#Check for NA in input variables
Newdata1<-na.omit(Newdata1)
Newdata<-Newdata1[, c("MW", "LogP", "LogS", "LogVP", "MP", "BP", "hCLAT", "OECD")]
IVdata<-IV[which(IV$Num %in% Newdata1$Num),]
#Define endpoint (LLLNA or human)
NewClass<-factor(IVdata[,"LLNA.POS.NEG"])
#Run SVM model and evaluate prediction
PredNew<-predict(LLNABinSVM, Newdata)
confusionMatrix(data=PredNew, reference=NewClass, positive="POS")

##########################################
#Mark overlap set chemicals
CE$ICCVAM_LLNA_Overlap<-0
CE$ICCVAM_LLNA_Overlap[CE$CASRN %in% intersect(CE$CASRN,LLNATraining$CASRN)]<-1


##########################################
##########################################
##Apply Human SVM model (Strickland et al. 2016)
##Read Cosmetic Europe in vitro data, physchem properties, OECD Toolbox predictions
# CE<-read.csv("CosEU_invitro_insilico_128chems_Jan2017.csv",header=TRUE)
# CE$CASRN<-trim(CE$CASRN)
# CE<-CE[order(CE$Num),]

#Format data (match feature names from ICCVAM model)
CE$Lys<-CE$DPRA.percLysdep
CE$Cys<-CE$DPRA.percCysdep
CE$avg.Lys.Cys<-CE$DPRA.mean
CE$DPRA<-CE$DPRA.Call
CE$Keratino<-CE$KS.Call
CE$hCLAT<-CE$hCLAT.Call

##########################################
#Read in vivo LLNA and human data
# IV<-read.csv("CosEU_invivo_128chems_Jan2017.csv",header=TRUE)
# IV$CASRN<-trim(IV$CASRN)
# IV<-IV[order(IV$Num),]

##Format data 
IV$human.POS.NEG<-"POS"
IV$human.POS.NEG[IV$human.Call==0]<-"NEG"

##########################################
#Build ICCVAM SVM Human model
#Read in the training data with 72 chemicals and 17 columns
HumanTraining <- read.table("ICCVAM-Human-Training-Jan23-2015.txt", header=T, sep="\t", as.is=T )
#Read in the test data with 24 chemicals and 17 columns
HumanTest <- read.table("ICCVAM-Human-Test-Jan23-2015.txt", header=T, sep="\t", as.is=T )
#Sensitizers vs Non-sensitizers 
#There are 21 negatives and 51 positives for training set
TrainingClass <- factor(c( rep("NEG", 21), rep("POS", 51)))
#There are 9 negatives and 15 positives for test set
TestClass <- factor(c( rep("NEG", 9), rep("POS", 15)))
#Model uses eight variables: LogP, OECD and assay values.
#Training set
traindata<-HumanTraining[,c("LogP", "avg.Lys.Cys",  "hCLAT", "Keratino", "OECD")]
#Test set
testdata<-HumanTest[,c("LogP", "avg.Lys.Cys",  "hCLAT", "Keratino", "OECD")]
#Set a weight for balancing the negative and positive classes.
wts <- 7/ table(TrainingClass)
#SVM model building
HumanBinSVMmodel <- svm(traindata, TrainingClass, cost = 100, gamma = 0.05, class.weights = wts)

##########################################
#Reproduce results from Strickland et al. 2016
#Predict the training set
PredSVMTraining<-predict(HumanBinSVMmodel, traindata)
table(PredSVMTraining, TrainingClass)
#Predict the test set
PredSVMTest<-predict(HumanBinSVMmodel, testdata)
table(PredSVMTest, TestClass)

##########################################
#Results for all chemicals (n=120)
#Create confusion matrix to compare in vitro call and LLNA result
Newdata1<-CE[, c("Num","CASRN","Chemical","LogP", "avg.Lys.Cys",  "hCLAT", "Keratino", "OECD")]
#Check for NA in input variables
Newdata1<-na.omit(Newdata1)
Newdata<-Newdata1[, c("LogP", "avg.Lys.Cys",  "hCLAT", "Keratino", "OECD")]
IVdata<-IV[which(IV$Num %in% Newdata1$Num),]
#Define endpoint (LLLNA or human)
NewClass<-factor(IVdata[,"human.POS.NEG"])
#Run SVM model and evaluate prediction
PredNew<-predict(HumanBinSVMmodel, Newdata)
confusionMatrix(data=PredNew, reference=NewClass, positive="POS")

#Add results to data file
HuSVM<-cbind(Newdata1,PredNew,NewClass)
names(HuSVM)[9:10]<-c("ICCVAM_SVM_Human_Prediction","Human_Class")
CE<-merge(CE,HuSVM,all.x=T)

##########################################
#Results for "test set" chemicals (n=63)
CEt<-CE[CE$CASRN %in% setdiff(CE$CASRN,HumanTraining$CASRN),]
IVt<-IV[IV$CASRN %in% setdiff(CE$CASRN,HumanTraining$CASRN),]
Newdata1<-CEt[, c("Num","CASRN","Chemical","LogP", "avg.Lys.Cys",  "hCLAT", "Keratino", "OECD")]
#Check for NA in input variables
Newdata1<-na.omit(Newdata1)
Newdata<-Newdata1[, c("LogP", "avg.Lys.Cys",  "hCLAT", "Keratino", "OECD")]
IVdata<-IV[which(IV$Num %in% Newdata1$Num),]
#Define endpoint (LLLNA or human)
NewClass<-factor(IVdata[,"human.POS.NEG"])
#Run SVM model and evaluate prediction
PredNew<-predict(HumanBinSVMmodel, Newdata)
confusionMatrix(data=PredNew, reference=NewClass, positive="POS")

##########################################
#Mark overlap set chemicals
CE$ICCVAM_Human_Overlap<-0
CE$ICCVAM_Human_Overlap[CE$CASRN %in% intersect(CE$CASRN,HumanTraining$CASRN)]<-1

