##########################################
## Cosmetics Europe Skin Sensitization
## Defined Approach Evaluation
## Nicole Kleinstreuer, Jason Pirone
## nicole.kleinstreuer@nih.gov
##
## Functions to reproduce Shiseido ANN model
##
##
############################################ 

library(RSNNS)

mlpFitWithScale <- function(Xtrain, yTrain, Xtest=NULL, yTest=NULL, maxit = 500,
                            params = c(0.02, 0.7), initFuncParams = c(-5,5),
                            stopIfOut=TRUE){
  
  ## Make sure the data sets are in the same order columnwise
  if(!is.null(Xtest)){
    nTestChems <- nrow(Xtest)
    Xtest <- Xtest[,colnames(Xtrain)]
  }
  
  ## Get the maximum and minimum values of each
  nInputs <- ncol(Xtrain)
  nTrainChems <- nrow(Xtrain)
  
  XtrainMin <- apply(Xtrain, 2, min)
  XtrainMax <- apply(Xtrain, 2, max)
  
  yTrainMin <- min(yTrain)
  yTrainMax <- max(yTrain)
  
  ## Check values of Xtest
  if(!is.null(Xtest)){
    
    lessThanMin <- any(Xtest < matrix(rep(XtrainMin, each=nTestChems), ncol=nInputs))
    greaterThanMax <- any(Xtest > matrix(rep(XtrainMax, each=nTestChems), ncol=nInputs))
    
    if((lessThanMin | greaterThanMax) & stopIfOut){
      stop("Error: At least one input value in the test data set is outside of the range of the training data.")
    }else if((lessThanMin | greaterThanMax) & !stopIfOut){
      warning("At least one input value in the test data set is outside of the range of the training data. Max and min were computed using the combined test and training data sets.")
      Xstack <- rbind(Xtrain, Xtest)
      yStack <- c(yTrain,yTest)
      
      Xmin <- apply(Xstack, 2, min)
      Xmax <- apply(Xstack, 2, max)
      
      yMin <- min(yStack)
      yMax <- max(yStack)
    }else{
      Xmin <- XtrainMin
      Xmax <- XtrainMax
      
      yMin <- yTrainMin
      yMax <- yTrainMax
    }
    
  }else{
    Xmin <- XtrainMin
    Xmax <- XtrainMax
    
    yMin <- yTrainMin
    yMax <- yTrainMax
  }
  
  
  XtrainScaled <- 2*(Xtrain-matrix(rep(Xmin,each=nTrainChems), ncol=nInputs)) /
    matrix(rep(Xmax-Xmin, each=nTrainChems), ncol=nInputs) - 1
  
  yTrainScaled <- (yTrain-yMin)/(yMax-yMin)
  
  if(!is.null(Xtest)){
    
    XtestScaled <- 2*(Xtest-matrix(rep(Xmin,each=nTestChems), ncol=nInputs)) /
      matrix(rep(Xmax-Xmin, each=nTestChems), ncol=nInputs) - 1
  }
  
  model <- mlp(XtrainScaled, yTrainScaled,
               size = c(5,2),
               maxit = maxit,
               initFunc = "Randomize_Weights",
               initFuncParams = initFuncParams,
               hiddenActFunc = "Act_Logistic",
               shufflePatterns = TRUE,
               updateFunc = "Topological_Order",
               learnFunc = "BackpropMomentum",
               learnFuncParams = params)
  
  if(!is.null(Xtest)){
    yPredScaled <- predict(model, XtestScaled)
    yPredOrigScale <- yPredScaled*(yMax-yMin) + yMin
    R <- cor(yPredOrigScale, yTest)
    RMSE <- sqrt(mean((yPredOrigScale - yTest)^2))
    yObs <- yTest
  }else{
    yPredScaled <- predict(model)
    yPredOrigScale <- yPredScaled*(yMax-yMin) + yMin
    R <- cor(yPredOrigScale, yTrain)
    RMSE <- sqrt(mean((yPredOrigScale - yTrain)^2))
    yObs <- yTrain
  }
  
  list(yObs = yObs,
       yPred = yPredOrigScale,
       R = R,
       RMSE = RMSE)
}

plotNetFit <- function(modelResults, AssayName="", topTitle=NULL){
  Obs <- modelResults$yObs
  Pred <- modelResults$yPred
  
  plot(Obs, Pred,
       pch = 19,cex =2, col = "steelblue",
       xlab = substitute(paste("Observed ",log[10],"(",nn,")"),list(nn=AssayName)),
       ylab = substitute(paste("Predicted ",log[10],"(",nn,")"),list(nn=AssayName)),
       main = paste0(topTitle,"\n",
                     "R=",round(modelResults$R,digits=3),
                     ", RMSE=",round(modelResults$RMSE,digits=3))
  )
  lmFit <- lm(Pred~Obs)
  abline(lmFit, lw=2,col="grey75")
}

